@@ -42,8 +42,6 @@ def get_fs(path):
         return Unknown('vfat32')
     elif re.search('minix', result, re.I):
         return Unknown('minix')
-    elif re.search('xfs', result, re.I):
-        return Unknown('xfs')
     elif re.search('jfs', result, re.I):
         return Unknown('jfs')
     elif re.search('reiserfs', result, re.I):
@@ -56,7 +54,7 @@ def get_fs(path):
 
 def get_filesystems():
     # NoFS has to be first
-    return [NoFS(), ext3(), ext2(), gfs2(), gfs2_clustered(), gfs(), gfs_clustered()]
+    return [NoFS(), ext4(), ext3(), ext2(), gfs2(), gfs2_clustered(), gfs(), gfs_clustered(), xfs()]
 
 
 class Filesystem:
@@ -146,6 +144,112 @@ class Unknown(Filesystem):
                             'unknown')
         
         
+class ext4(Filesystem):
+    def __init__(self):
+        creatable = self.check_path('/sbin/mkfs.ext4')
+        mountable = self.check_mountable('ext4', 'ext4')
+        resize_offline = self.check_paths(['/sbin/e2fsck', '/sbin/resize2fs'])
+        extend_online = self.check_path('/sbin/resize2fs')
+
+        Filesystem.__init__(self, 'Ext4', creatable, True, mountable,
+                            extend_online, resize_offline, False, resize_offline,
+                            'ext4')
+
+
+    def probe(self, path):
+        result = execWithCapture("/usr/bin/file", ['/usr/bin/file', '-s', '-L', path])
+        if re.search('ext4', result, re.I):
+            return True
+
+    # Note, you may need a test for new-enough e2fsprogs for ext4
+    def create(self, path):
+        args = list()
+        args.append("/sbin/mkfs")
+        args.append("-t")
+        args.append('ext4')
+        args.append(path)
+        cmdstr = ' '.join(args)
+        msg = CREATING_FS % (self.name)
+        o,e,r = execWithCaptureErrorStatusProgress("/sbin/mkfs", args, msg)
+        if r != 0:
+            raise CommandError('FATAL', FSCREATE_FAILURE % (cmdstr,e))
+
+    def extend_online(self, dev_path):
+        cmd = '/sbin/resize2fs'
+        args = [cmd, dev_path]
+        cmdstr = ' '.join(args)
+        msg = RESIZING_FS % (self.name)
+        o, e, s = execWithCaptureErrorStatusProgress(cmd, args, msg)
+        if s != 0:
+            raise CommandError('FATAL', FSRESIZE_FAILURE % (cmdstr, e))
+
+    def reduce_online(self, dev_path, new_size_bytes):
+        # not supported
+        raise 'NOT supported'
+
+    def extend_offline(self, dev_path):
+        # first check fs (resize2fs requirement)
+        args = list()
+        args.append('/sbin/e2fsck')
+        args.append('-f')
+        args.append('-p') # repair fs
+        args.append(dev_path)
+        cmdstr = ' '.join(args)
+        msg = CHECKING_FS % self.name
+        o,e,r = execWithCaptureErrorStatusProgress('/sbin/e2fsck', args, msg)
+        if not (r==0 or r==1):
+            raise CommandError('FATAL', FSCHECK_FAILURE % (cmdstr,e))
+
+        args = list()
+        args.append('/sbin/resize2fs')
+        args.append(dev_path)
+        cmdstr = ' '.join(args)
+        msg = RESIZING_FS % self.name
+        o,e,r = execWithCaptureErrorStatusProgress('/sbin/resize2fs', args, msg)
+        if r != 0:
+            raise CommandError('FATAL', FSRESIZE_FAILURE % (cmdstr,e))
+
+    def reduce_offline(self, dev_path, new_size_bytes):
+        # first check fs (resize2fs requirement)
+        args = list()
+        args.append('/sbin/e2fsck')
+        args.append('-f')
+        args.append('-p') # repair fs
+        args.append(dev_path)
+        cmdstr = ' '.join(args)
+        msg = CHECKING_FS % self.name
+        o,e,r = execWithCaptureErrorStatusProgress('/sbin/e2fsck', args, msg)
+        if not (r==0 or r==1):
+            raise CommandError('FATAL', FSCHECK_FAILURE % (cmdstr,e))
+
+        new_size_kb = new_size_bytes / 1024
+        args = list()
+        args.append('/sbin/resize2fs')
+        args.append(dev_path)
+        args.append(str(new_size_kb) + 'K')
+        cmdstr = ' '.join(args)
+        msg = RESIZING_FS % (self.name)
+        o,e,r = execWithCaptureErrorStatusProgress('/sbin/resize2fs', args, msg)
+        if r != 0:
+            raise CommandError('FATAL', FSRESIZE_FAILURE % (cmdstr,e))
+
+    def get_label(self, devpath):
+        args = ['/sbin/tune2fs']
+        args.append('-l')
+        args.append(devpath)
+        o, r = execWithCaptureStatus('/sbin/tune2fs', args)
+        if r == 0:
+            lines = o.splitlines()
+            for line in lines:
+                if re.search('volume name', line, re.I):
+                    words = line.split()
+                    label = words[len(words) - 1]
+                    if re.match('<none>', label, re.I):
+                        return None
+                    else:
+                        return label
+        return None
+
 class ext3(Filesystem):
     def __init__(self):
         creatable = self.check_path('/sbin/mkfs.ext3')
@@ -192,7 +296,7 @@ class ext3(Filesystem):
         try:
             cmd = '/sbin/resize2fs'
             o, e, s = execWithCaptureErrorStatus(cmd, [cmd])
-            if (s == 0 or s == 1) and float(e.strip().split()[1]) >= 1.39:
+            if (s in [0, 1] and map(int, e.strip().split()[1].split('.')) >= [1, 39]):
                 return (True, cmd)
         except:
             pass
@@ -652,6 +756,66 @@ class gfs_clustered(Filesystem):
                         return v
         return None
 
+class xfs(Filesystem):
+    def __init__(self):
+        creatable = self.check_path('/sbin/mkfs.xfs')
+        mountable = self.check_mountable('xfs', 'xfs')
+        extend_online = self.check_path('/usr/sbin/xfs_growfs')
+
+        Filesystem.__init__(self, 'XFS', creatable, True, mountable,
+                            extend_online, False, False, False,
+                            'xfs')
+
+    def probe(self, path):
+        result = execWithCapture("/usr/bin/file", ['/usr/bin/file', '-s', '-L', path])
+        if re.search('SGI XFS', result, re.I):
+            return True
+
+    def create(self, path):
+        args = list()
+        args.append("/sbin/mkfs")
+        args.append("-t")
+        args.append('xfs')
+        args.append(path)
+        cmdstr = ' '.join(args)
+        msg = CREATING_FS % (self.name)
+        o,e,r = execWithCaptureErrorStatusProgress("/sbin/mkfs", args, msg)
+        if r != 0:
+            raise CommandError('FATAL', FSCREATE_FAILURE % (cmdstr,e))
+
+    def extend_online(self, dev_path):
+        cmd = '/usr/sbin/xfs_growfs'
+        args = [cmd, dev_path]
+        cmdstr = ' '.join(args)
+        msg = RESIZING_FS % (self.name)
+        o, e, s = execWithCaptureErrorStatusProgress(cmd, args, msg)
+        if s != 0:
+            raise CommandError('FATAL', FSRESIZE_FAILURE % (cmdstr, e))
+
+    def reduce_online(self, dev_path, new_size_bytes):
+        # not supported
+        raise 'NOT supported'
+
+    def extend_offline(self, dev_path):
+        # not supported
+        raise 'NOT supported'
+
+    def reduce_offline(self, dev_path, new_size_bytes):
+        # not supported
+        raise 'NOT supported'
+
+    def get_label(self, devpath):
+        args = ['/usr/sbin/xfs_admin']
+        args.append('-l')
+        args.append(devpath)
+        o, r = execWithCaptureStatus('/usr/sbin/xfs_admin', args)
+        if r == 0:
+            words = o.split()
+            label = re.sub('\"', '', words[len(words) - 1])
+            if label:
+                return label
+            return None
+        return None
     
 
 
