@@ -22,23 +22,25 @@ FSTAB_TMP = '/etc/fstab.tmp.system-config-lvm'
     
 def add(dev_path_old, dev_path, mnt_point, fstype, options='defaults', dump='1', fsck='2'):
     line = dev_path + '\t\t' + mnt_point + '\t\t' + fstype + '\t' + options
-    line = line + '\t' + dump + ' ' + fsck + '\n'
-    
-    fstab = __remove(dev_path_old)
-    fstab.write(line)
-    fstab.close()
+    line = line + '\t' + dump + ' ' + fsck
     
+    fstab = __remove_and_replace(dev_path_old, line)
+    fstab.close()    
     os.rename(FSTAB_TMP, FSTAB)
     
 def remove(mnt_device):
-    fstab = __remove(mnt_device)
+    fstab = __remove_and_replace(mnt_device, None)
     fstab.close()
     os.rename(FSTAB_TMP, FSTAB)
     
-def __remove(mnt_device):
+def __remove_and_replace(mnt_device, new_line):
+    added = False
+
     fstab = open(FSTAB, 'r')
     lines = fstab.readlines()
     fstab.close()
+
+    paths = get_all_paths(mnt_device)
     
     fstab_new = open(FSTAB_TMP, 'w')
     for line in lines:
@@ -59,12 +61,18 @@ def __remove(mnt_device):
             fstab_new.write(line + '\n')
             continue
         
-        if words[DEVICE] == mnt_device:
-            # line needs to be removed
+        if words[DEVICE] in paths:
+            # line needs to be removed/replaced
+            if (new_line != None) and (added == False):
+              fstab_new.write(new_line + '\n')
+              added = True
             pass
         else:
             fstab_new.write(line + '\n')
             
+    if (new_line != None) and (added == False):
+      fstab_new.write(new_line + '\n')
+            
     return fstab_new
 
 
@@ -76,11 +84,7 @@ def get_mount_options(dev_path):
     if dev_path == None:
         return (None, None, None, None, None, None)
     
-    paths = [dev_path]
-    follow_links_to_target(dev_path, paths)
-    label = Filesystem.get_fs(dev_path).get_label(dev_path)
-    if label != None:
-        paths.append('LABEL=' + label)
+    paths = get_all_paths(dev_path)
     
     fstab = open(FSTAB, 'r')
     lines = fstab.readlines()
@@ -104,3 +108,12 @@ def get_mount_options(dev_path):
             return words
 
     return (None, None, None, None, None, None)
+
+def get_all_paths(dev_path):
+    paths = [dev_path]
+    follow_links_to_target(dev_path, paths)
+    label = Filesystem.get_fs(dev_path).get_label(dev_path)
+    if label != None:
+        paths.append('LABEL=' + label)
+
+    return paths
