@@ -235,13 +235,13 @@ class Volume_Tab_View:
                           OBJ_COL, vg)
             phys_iter = treemodel.append(vg_child_iter)
             log_iter = treemodel.append(vg_child_iter)
-            pview_string = vg_name + "<span foreground=\"#ED1C2A\"><i>  " + PHYSICAL_VIEW + "</i></span>"
+            pview_string = "<span foreground=\"#ED1C2A\"><i>  " + PHYSICAL_VIEW + "</i></span>"
             treemodel.set(phys_iter,
                           NAME_COL, pview_string,
                           TYPE_COL, VG_PHYS_TYPE,
                           PATH_COL, vg_name, 
                           OBJ_COL, vg)
-            lview_string = vg_name + "<span foreground=\"#43ACF6\"><i>  " + LOGICAL_VIEW + "</i></span>"
+            lview_string = "<span foreground=\"#43ACF6\"><i>  " + LOGICAL_VIEW + "</i></span>"
             treemodel.set(log_iter,
                           NAME_COL, lview_string,
                           TYPE_COL, VG_LOG_TYPE,
