@@ -548,7 +548,9 @@ class lvm_model:
       origin = words[LV_SNAP_ORIGIN_IDX].strip()
       if origin != '':
         # snapshot
-        usage = float(words[LV_SNAP_PERCENT_IDX].strip())
+        usage = float(0)
+        if (words[LV_SNAP_PERCENT_IDX] != "" ):
+          usage = float(words[LV_SNAP_PERCENT_IDX].strip())
         lv.set_snapshot_info(origin, usage) # set name for now, real LV will get set at __link_snapshots()
       
     # all LVs created  
@@ -1049,6 +1051,8 @@ class lvm_model:
       possible_path = words[0]
       if possible_path in paths:
         return words[1]
+      if follow_links_to_target(possible_path, []) in paths:
+        return words[1]
     return None
   
   def is_mirroring_supported(self):
